import Vue from 'vue';

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();

/**
 * Layer last z-index storage
 * @private
 * @type {number}
 */
var zIndex = 8000;

/**
 * Layer class
 */

var Layer = function () {
  function Layer() {
    classCallCheck(this, Layer);

    /**
     * Specifily vue element
     * @type {Object}
     */
    this.element = undefined;

    /**
     * vueComponent within a wrapper
     * @type {DOM}
     */
    this.wrapper = undefined;

    /**
     * Forget
     * @type {boolean}
     */
    this.forget = undefined;
  }

  /**
   * Create a new layer
   * @param  {Object} propsData  - Vue propsData
   * @return {Layer}             - Layer
   */


  createClass(Layer, [{
    key: 'createComponent',


    /**
     * create vueComponent
     * @param  {Object} propsData  - Vue propsData
     * @return {Layer}
     */
    value: function createComponent(propsData) {
      if (typeof this.element === 'undefined') {
        throw new Error('The this.element should be a vue element.');
      }

      var VueComponent = Vue.extend(this.element);
      var vueComponent = new VueComponent({
        propsData: this.mergePropsData(propsData)
      }).$mount();

      this.listen(vueComponent);
      this.append(vueComponent);
      this.vm = vueComponent;
      return this;
    }

    /**
     * Merge propsData default value
     * @protected
     * @param  {Object} propsData  - vue propsData
     * @return {Object}
     */

  }, {
    key: 'mergePropsData',
    value: function mergePropsData(propsData) {
      return Object.assign({}, {
        zIndex: this.generateZindex()
      }, propsData);
    }

    /**
     * Generate zIndex
     * @private
     * @return {number}
     */

  }, {
    key: 'generateZindex',
    value: function generateZindex() {
      zIndex += 10;
      return zIndex;
    }

    /**
     * Listen vueComponent's update:visible event
     * @protected
     * @param  {VueComponent} vueComponent
     * @return {Layer}
     */

  }, {
    key: 'listen',
    value: function listen(vueComponent) {
      var _this = this;

      vueComponent.$on('update:visible', function (visible) {
        visible ? _this.open() : _this.destroy();
      });
      return this;
    }

    /**
     * Wrap vueComponent and append to body
     * @param  {VueComponent} vueComponent
     * @return {Layer}
     */

  }, {
    key: 'append',
    value: function append(vueComponent) {
      this.wrapper = document.createElement('div');
      this.wrapper.id = 'mn-popup-#' + vueComponent._uid;
      document.body.appendChild(this.wrapper);
      this.wrapper.appendChild(vueComponent.$el);
      return this;
    }

    /**
     * Destroy vueComponent and remove wrapper from body
     * @protected
     * @return {Layer}
     */

  }, {
    key: 'remove',
    value: function remove() {
      if (typeof this.vm !== 'undefined') {
        this.vm.$destroy();
        this.removeWrapper();
      }
      this.vm = undefined;
      this.wrapper = undefined;
      return this;
    }

    /**
     * Remove wrapper
     * @protected
     * @return {Layer}
     */

  }, {
    key: 'removeWrapper',
    value: function removeWrapper() {
      if (this.wrapper) {
        document.body.removeChild(this.wrapper);
      }
      return this;
    }

    /**
     * Show vueComponent
     * @return {Layer}
     */

  }, {
    key: 'show',
    value: function show() {
      if (!this.vm) throw new Error('vueComponent is already destroyed.');
      this.vm.$props.visible = true;
      return this;
    }

    /**
     * Hide vueComponent, and then destroy it.
     * @return {Promise}
     */

  }, {
    key: 'destroy',
    value: function destroy() {
      var _this2 = this;

      return new Promise(function (resolve) {
        if (_this2.vm) {
          _this2.vm.$props.visible = false;

          setTimeout(function () {
            _this2.remove();
            resolve(_this2);
          }, 1500);
        }

        resolve(_this2);
      });
    }

    /**
     * Wrap vm.$on event, and return this
     * @param  {string}   name       - event name from vueComponent
     * @param  {Function} callback   - pass callback to vueComponent
     * @return {Layer}
     */

  }, {
    key: 'on',
    value: function on(name, callback) {
      this.vm.$on(name, callback);
      return this;
    }
  }], [{
    key: 'create',
    value: function create(propsData) {
      var instance = new this();
      return instance.createComponent(propsData);
    }
  }]);
  return Layer;
}();

var layerMixin = {
  props: {
    /**
     * Control layer show or hide
     */
    visible: Boolean,
    /**
     * Set transition name
     */
    transition: String,
    /**
     * Define z-index
     */
    zIndex: {
      type: Number,
      default: 2000
    }
  },
  methods: {
    show: function show() {
      /**
       * @event update:visible
       * @property {Boolean} visible - show visible
       */
      this.$emit('update:visible', true);
    },
    hide: function hide() {
      /**
       * @event update:visible
       * @property {Boolean} visible - hide visible
       */
      this.$emit('update:visible', false);
    }
  }
};

export { Layer, layerMixin };
